@@ -48,7 +48,6 @@
 
 typedef enum {
   GLADE_BUTTON_NORMAL,
-  GLADE_BUTTON_TOOLBAR,
   GLADE_BUTTON_DIALOG,
   GLADE_BUTTON_GNOME_DIALOG
 } GladeButtonType;
@@ -63,6 +62,7 @@ static GbWidget gbwidget;
 static gchar *StockButton = "GtkButton::stock_button";
 static gchar *Label = "GtkButton::label";
 static gchar *Icon = "GtkButton::icon";
+static gchar *FocusOnClick = "GtkButton::focus_on_click";
 
 /* This is only used for normal/stock buttons, not special toolbar buttons,
    as the toolbar has its own relief setting. */
@@ -95,23 +95,10 @@ gb_button_new (GbWidgetNewData * data)
 {
   GtkWidget *new_widget;
 
-  /* SPECIAL CODE: to handle toolbar buttons. */
-  MSG1 ("IN gb_button_new parent:%s",
-	data->parent ? gtk_widget_get_name (data->parent) : "NULL");
-  if (data->parent && GTK_IS_TOOLBAR (data->parent))
-    {
-      MSG ("   creating toolbar button...");
-      new_widget = gb_toolbar_new_toolbar_button (data,
-						  GTK_TOOLBAR_CHILD_BUTTON);
-      if (new_widget)
-	return new_widget;
-    }
-
   if (data->action == GB_CREATING)
     new_widget = gtk_button_new_with_label (data->name);
   else
     {
-      MSG ("   creating normal button with placeholder...");
       new_widget = gtk_button_new ();
       gtk_container_add (GTK_CONTAINER (new_widget), editor_new_placeholder());
     }
@@ -146,20 +133,17 @@ gb_button_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
 	  "Select one of the standard responses or enter a positive integer value"),
 		      response_list);
   g_list_free (response_list);
+
+  property_add_bool (FocusOnClick, _("Focus On Click:"), _("If the button grabs focus when it is clicked"));
 }
 
 
-/* Returns the type of the button - NORMAL, TOOLBAR or DIALOG. Note that
-   TOOLBAR means it is a special toolbar button. Toolbars can also contain
-   NORMAL buttons. */
+/* Returns the type of the button - NORMAL, DIALOG or GNOME_DIALOG. */
 GladeButtonType
 gb_button_get_button_type (GtkWidget *widget)
 {
   gchar *child_name;
 
-  if (gb_toolbar_is_toolbar_button (widget))
-    return GLADE_BUTTON_TOOLBAR;
-
   if (widget->parent)
     {
       child_name = gb_widget_get_child_name (widget->parent);
@@ -201,10 +185,6 @@ gb_button_normal_find_child_widgets (GtkWidget *widget,
   /* Check if it has just a GtkLabel child. */
   if (GTK_IS_LABEL (child))
     {
-      /* If it is a GbWidget we don't handle its properties here. */
-      if (GB_IS_GB_WIDGET (child))
-	return FALSE;
-
       *label = child;
       return TRUE;
     }
@@ -671,310 +651,6 @@ gb_button_normal_write_source (GtkWidget * widget,
 }
 
 
-/*
- * SPECIAL TOOLBAR BUTTONS. GtkButton/GtkToggleButton/GtkRadioButton.
- */
-
-/* This finds the icon and/or label widgets within a toolbar button.
-   If icon_return and label_return are non-NULL the pointers are placed there.
-   it returns TRUE if it finds them OK. */
-gboolean
-gb_button_toolbar_find_child_widgets (GtkWidget *widget,
-				      GtkWidget **label,
-				      GtkWidget **icon)
-{
-  GtkWidget *child;
-  GList *box_children;
-
-  *label = *icon = NULL;
-
-  child = GTK_BIN (widget)->child;
-  g_return_val_if_fail (child != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_BOX (child), FALSE);
-
-  box_children = GTK_BOX (child)->children;
-  g_return_val_if_fail (g_list_length (box_children) == 2, FALSE);
-
-  /* Note that the label is added first in gtktoolbar.c which is why
-     it is first in the list. */
-  *label = ((GtkBoxChild*)box_children->data)->widget;
-  *icon = ((GtkBoxChild*)box_children->next->data)->widget;
-
-  g_return_val_if_fail (GTK_IS_LABEL (*label), FALSE);
-  g_return_val_if_fail (!GB_IS_GB_WIDGET (*label), FALSE);
-  g_return_val_if_fail (GTK_IS_IMAGE (*icon), FALSE);
-  g_return_val_if_fail (!GB_IS_GB_WIDGET (*icon), FALSE);
-
-  return TRUE;
-}
-
-
-static void
-gb_button_toolbar_get_properties (GtkWidget * widget,
-				  GbWidgetGetArgData * data,
-				  gchar *stock_id_p,
-				  gchar *label_p,
-				  gchar *icon_p)
-{
-  gchar *stock_id, *label_text;
-  gboolean label_sensitive = TRUE;
-  gboolean icon_sensitive = TRUE;
-  GtkWidget *label, *icon;
-
-  /* We use the icon size from the parent toolbar. */
-  if (data->action == GB_SHOWING)
-    {
-      GtkIconSize icon_size;
-
-      icon_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (widget->parent));
-      property_set_stock_item_icon_size (stock_id_p, icon_size);
-      property_set_icon_size (icon_p, icon_size);
-    }
-
-  stock_id = gtk_object_get_data (GTK_OBJECT (widget), GladeButtonStockIDKey);
-  /* In the XML the stock item name is actually saved as the label property,
-     and the use_stock property specifies that it is a stock item.
-     Also, use_underline needs to be set for non-stock items.
-     Note that currently GTK+ doesn't really have property support for special
-     toolbar buttons, so it is all hacked, here and in libglade. */
-  if (data->action == GB_SAVING)
-    {
-      if (stock_id)
-	{
-	  gb_widget_output_stock_item (data, "label", stock_id);
-	  gb_widget_output_bool (data, "use_stock", TRUE);
-	}
-    }
-  else
-    {
-      gb_widget_output_stock_item (data, stock_id_p, stock_id);
-    }
-
-
-  if (stock_id)
-    {
-      label_sensitive = FALSE;
-      icon_sensitive = FALSE;
-    }
-  else
-    {
-      gchar *icon_name = NULL;
-
-      /* A special toolbar button must have an icon and a label. */
-      if (!gb_button_toolbar_find_child_widgets (widget, &label, &icon))
-	{
-	  g_warning ("Couldn't find special toolbar button children");
-	  return;
-	}
-
-	label_text = (gchar*) gtk_label_get_label (GTK_LABEL (label));
-	gb_widget_output_translatable_text (data, label_p, label_text);
-
-	/* We always save use_underline as TRUE, though we don't load it. */
-	if (data->action == GB_SAVING)
-	  gb_widget_output_bool (data, "use_underline", TRUE);
-
-	icon_name = gtk_object_get_data (GTK_OBJECT (icon), GladeIconKey);
-	/* When saving, we save stock icons as "stock_pixmap". */
-	if (data->action == GB_SAVING
-	    && glade_util_check_is_stock_id (icon_name))
-	  gb_widget_output_icon (data, "stock_pixmap", icon_name);
-	else
-	  gb_widget_output_icon (data, icon_p, icon_name);
-    }
-
-  if (data->action == GB_SHOWING)
-    {
-      if (!label_sensitive)
-	gb_widget_output_text (data, label_p, "");
-      property_set_sensitive (label_p, label_sensitive);
-
-      if (!icon_sensitive)
-	  gb_widget_output_pixmap_filename (data, icon_p, "");
-      property_set_sensitive (icon_p, icon_sensitive);
-    }
-}
-
-
-static void
-gb_button_toolbar_set_icon (GtkWidget *widget,
-			    GbWidgetSetArgData * data,
-			    GtkWidget *icon_widget,
-			    gchar *icon_name)
-{
-  gchar *old_icon_name;
-
-  if (icon_name && !*icon_name)
-    icon_name = NULL;
-
-  /* Remove the old icon_name stored in the widget data, and remove the
-     pixmap from the project, if necessary. */
-  old_icon_name = gtk_object_get_data (GTK_OBJECT (icon_widget),
-				       GladeIconKey);
-  glade_project_remove_pixmap (data->project, old_icon_name);
-
-  if (glade_util_check_is_stock_id (icon_name))
-    {
-      GtkIconSize icon_size;
-
-      icon_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (widget->parent));
-      gtk_image_set_from_stock (GTK_IMAGE (icon_widget), icon_name,
-				icon_size);
-    }
-  else if (icon_name)
-    {
-      gtk_image_set_from_file (GTK_IMAGE (icon_widget), icon_name);
-      glade_project_add_pixmap (data->project, icon_name);
-    }
-  else
-    {
-      GbWidget *pixmap_gbwidget;
-
-      pixmap_gbwidget = gb_widget_lookup_class ("GtkImage");
-      if (pixmap_gbwidget)
-	gtk_image_set_from_pixmap (GTK_IMAGE (icon_widget),
-				   pixmap_gbwidget->gdkpixmap,
-				   pixmap_gbwidget->mask);
-      else
-	g_warning ("Couldn't find GtkPixmap data");
-    }
-
-  gtk_object_set_data_full (GTK_OBJECT (icon_widget), GladeIconKey,
-			    g_strdup (icon_name),
-			    icon_name ? g_free : NULL);
-}
-
-
-static void
-gb_button_toolbar_set_stock_id (GtkWidget *widget,
-				GbWidgetSetArgData * data,
-				gchar *stock_id,
-				gchar *label_p,
-				gchar *icon_p,
-				GtkWidget *label_widget,
-				GtkWidget *icon_widget)
-{
-  gboolean is_stock_item = FALSE;
-  const gchar *label_text = "";
-
-  if (stock_id && stock_id[0])
-    is_stock_item = TRUE;
-
-  if (is_stock_item)
-    {
-      GtkStockItem item;
-      GtkIconSize icon_size;
-
-      icon_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (widget->parent));
-      gtk_stock_lookup (stock_id, &item);
-
-      gtk_image_set_from_stock (GTK_IMAGE (icon_widget), stock_id, icon_size);
-      gtk_label_set_text_with_mnemonic (GTK_LABEL (label_widget), item.label);
-      gtk_object_set_data_full (GTK_OBJECT (widget), GladeButtonStockIDKey,
-				g_strdup (stock_id), g_free);
-    }
-  else
-    {
-      /* Change the button back to a normal toolbar button with a label and
-	 an icon. */
-      GbWidget *pixmap_gbwidget;
-
-      pixmap_gbwidget = gb_widget_lookup_class ("GtkImage");
-      if (pixmap_gbwidget)
-	gtk_image_set_from_pixmap (GTK_IMAGE (icon_widget),
-				   pixmap_gbwidget->gdkpixmap,
-				   pixmap_gbwidget->mask);
-      else
-	g_warning ("Couldn't find GtkPixmap data");
-
-      label_text = gtk_widget_get_name (widget);
-      gtk_label_set_text_with_mnemonic (GTK_LABEL (label_widget),
-					label_text);
-
-      gtk_object_set_data (GTK_OBJECT (widget), GladeButtonStockIDKey, NULL);
-    }
-
-  /* If the widget's properties are displayed, we update the sensitivity of
-     the label and icon, according to whether a stock item is selected. */
-  if (data->action == GB_APPLYING && property_get_widget () == widget)
-    {
-      property_set_sensitive (label_p, !is_stock_item);
-      property_set_sensitive (icon_p, !is_stock_item);
-
-      /* Turn off auto-apply, and set the label. */
-      property_set_auto_apply (FALSE);
-      property_set_text (label_p, label_text);
-      property_set_filename (icon_p, "");
-      property_set_auto_apply (TRUE);
-    }
-}
-
-
-void
-gb_button_toolbar_set_properties	(GtkWidget * widget,
-					 GbWidgetSetArgData * data,
-					 gchar *stock_id_p,
-					 gchar *label_p,
-					 gchar *icon_p)
-{
-  gchar *stock_id = NULL, *label, *icon_name;
-  gboolean apply_stock_id = FALSE, apply_label;
-  gboolean free_label = FALSE;
-  GtkWidget *label_widget, *icon_widget;
-
-  if (!gb_button_toolbar_find_child_widgets (widget, &label_widget,
-					     &icon_widget))
-    {
-      g_warning ("Couldn't find special toolbar button children");
-      return;
-    }      
-
-  label = gb_widget_input_text (data, label_p);
-  apply_label = data->apply;
-  if (data->action == GB_APPLYING)
-    free_label = TRUE;
-
-  /* When loading, if "use_stock" is TRUE, then the label is the stock item. */
-  if (data->action == GB_LOADING)
-    {
-      gboolean is_stock_item = gb_widget_input_bool (data, "use_stock");
-      if (is_stock_item)
-	{
-	  stock_id = label;
-	  apply_stock_id = apply_label;
-	}
-    }
-  else
-    {
-      stock_id = gb_widget_input_stock_item (data, stock_id_p);
-      apply_stock_id = data->apply;
-    }
-
-  if (apply_stock_id)
-    {
-      gb_button_toolbar_set_stock_id (widget, data, stock_id, label_p, icon_p,
-				      label_widget, icon_widget);
-      goto out;
-    }
-
-  icon_name = gb_widget_input_icon (data, icon_p);
-  /* When loading we also check for "stock_pixmap". */
-  if (!data->apply && data->action == GB_LOADING)
-      icon_name = gb_widget_input_icon (data, "stock_pixmap");
-  if (data->apply)
-    gb_button_toolbar_set_icon (widget, data, icon_widget, icon_name);
-
-  if (apply_label)
-    gtk_label_set_text_with_mnemonic (GTK_LABEL (label_widget),
-				      label ? label : "");
-						   
- out:
-
-  if (free_label)
-    g_free (label);
-}
-
-
 void
 gb_button_find_radio_group (GtkWidget *widget, GladeFindGroupData *find_data)
 {
@@ -997,140 +673,6 @@ gb_button_find_radio_group (GtkWidget *widget, GladeFindGroupData *find_data)
 }
 
 
-static void
-gb_button_toolbar_write_source (GtkWidget * widget,
-				GbWidgetWriteSourceData * data)
-{
-  GladeWidgetData *wdata = data->widget_data;
-  GtkWidget *label_widget, *icon_widget;
-  gchar *parent_name, *label_text, *type, *radio_widget_name = NULL;
-  gchar *stock_id, *icon_name = NULL;
-  GType widget_type;
-  gboolean has_icon = FALSE;
-
-  g_return_if_fail (widget->parent != NULL);
-  parent_name = (gchar*) gtk_widget_get_name (widget->parent);
-  parent_name = source_create_valid_identifier (parent_name);
-
-  widget_type = G_OBJECT_TYPE (widget);
-  if (widget_type == GTK_TYPE_BUTTON)
-    type = "GTK_TOOLBAR_CHILD_BUTTON";
-  else if (widget_type == GTK_TYPE_TOGGLE_BUTTON)
-    type = "GTK_TOOLBAR_CHILD_TOGGLEBUTTON";
-  else
-    {
-      GtkWidget *component;
-      GladeFindGroupData find_data;
-
-      type = "GTK_TOOLBAR_CHILD_RADIOBUTTON";
-      component = glade_util_get_toplevel (widget);
-      find_data.group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (widget));
-      find_data.found_widget = NULL;
-      gb_button_find_radio_group (component, &find_data);
-
-      if (find_data.found_widget && find_data.found_widget != widget)
-	radio_widget_name = source_create_valid_identifier (gtk_widget_get_name (find_data.found_widget));
-    }
-
-  /* If it should start a new group, add a space. */
-  if (gb_toolbar_get_new_toolbar_group (widget->parent, widget))
-    source_add (data, "  gtk_toolbar_append_space (GTK_TOOLBAR (%s));\n\n",
-		parent_name);
-
-  gb_button_toolbar_find_child_widgets (widget, &label_widget, &icon_widget);
-
-  stock_id = gtk_object_get_data (GTK_OBJECT (widget), GladeButtonStockIDKey);
-  if (stock_id)
-    {
-      if (G_OBJECT_TYPE (widget) == GTK_TYPE_BUTTON)
-	{
-	  source_add (data,
-		      "  %s = gtk_toolbar_insert_stock (GTK_TOOLBAR (%s),\n"
-		      "                                %s,\n",
-		      data->wname,
-		      parent_name,
-		      source_make_string (stock_id, FALSE));
-	  source_add (data,
-		      "                                %s,\n"
-		      "                                NULL, NULL, NULL, -1);\n",
-		      wdata->tooltip ? source_make_string (wdata->tooltip, data->use_gettext) : "NULL");
-
-	  g_free (parent_name);
-	  return;
-	}
-      else
-	{
-	  /* GtkToolbar doesn't have builtin support for stock toggle & radio
-	     items, so we create the label by looking up the stock id, and
-	     we create the icon just like a stock icon below. */
-	  source_ensure_decl (data, "  GtkStockItem tmp_stock_item;\n");
-
-	  /* We set the stock item label to "" just in case it isn't found. */
-	  source_add (data,
-		      "  tmp_stock_item.label = "";\n"
-		      "  gtk_stock_lookup (%s, &tmp_stock_item);\n",
-		      source_make_string (stock_id, FALSE));
-
-	  label_text = g_strdup ("tmp_stock_item.label");
-
-	  /* We use the stock_id for the icon below. */
-	  icon_name = stock_id;
-	}
-    }
-  else
-    {
-      gchar *label = glade_util_get_label_text (label_widget);
-      label_text = g_strdup (source_make_string (label, data->use_gettext));
-      g_free (label);
-
-      icon_name = gtk_object_get_data (GTK_OBJECT (icon_widget), GladeIconKey);
-    }
-
-  /* Make sure the tmp_toolbar_icon widget is declared. */
-  if (icon_name && *icon_name)
-    {
-      has_icon = TRUE;
-      source_ensure_decl (data, "  GtkWidget *tmp_toolbar_icon;\n");
-
-      if (glade_util_check_is_stock_id (icon_name))
-	{
-	  source_add (data,
-		      "  tmp_toolbar_icon = gtk_image_new_from_stock (\"%s\", gtk_toolbar_get_icon_size (GTK_TOOLBAR (%s)));\n",
-		      icon_name, parent_name);
-	}
-      else
-	{
-	  source_create_pixmap (data, "tmp_toolbar_icon", icon_name);
-	}
-    }
-
-
-  source_add (data,
-	      "  %s = gtk_toolbar_append_element (GTK_TOOLBAR (%s),\n"
-	      "                                %s,\n"
-	      "                                %s,\n"
-	      "                                %s,\n"
-	      "                                %s, NULL,\n"
-	      "                                %s, NULL, NULL);\n",
-	      data->wname, parent_name, type,
-	      radio_widget_name ? radio_widget_name : "NULL",
-	      label_text,
-	      wdata->tooltip ? source_make_string (wdata->tooltip, data->use_gettext) : "NULL",
-	      has_icon ? "tmp_toolbar_icon" : "NULL");
-
-  /* FIXME: GTK+ doesn't set "use_underline" for the label, so we set it here.
-     Though I'm not really sure toolbar buttons should have mnemonic accels,
-     but the stock items do so we are consistent. */
-  source_add (data,
-	      "  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (%s)->children)->data))->label), TRUE);\n",
-	      parent_name);
-
-  g_free (label_text);
-  g_free (radio_widget_name);
-  g_free (parent_name);
-}
-
-
 /*
  * GNOME DIALOG BUTTONS. GtkButton only.
  */
@@ -1645,7 +1187,8 @@ gb_button_get_standard_properties (GtkWidget * widget,
 				   gchar *stock_id_p,
 				   gchar *label_p,
 				   gchar *icon_p,
-				   gchar *relief_p)
+				   gchar *relief_p,
+				   gchar *focus_on_click_p)
 {
   GladeButtonType button_type;
   gboolean relief_visible = TRUE, icon_file_selection = TRUE;
@@ -1659,11 +1202,6 @@ gb_button_get_standard_properties (GtkWidget * widget,
       gb_button_normal_get_properties (widget, data, stock_id_p, label_p,
 				       icon_p);
       break;
-    case GLADE_BUTTON_TOOLBAR:
-      gb_button_toolbar_get_properties (widget, data, stock_id_p, label_p,
-					icon_p);
-      relief_visible = FALSE;
-      break;
     case GLADE_BUTTON_DIALOG:
       gb_button_normal_get_properties (widget, data, stock_id_p, label_p,
 				       icon_p);
@@ -1691,6 +1229,10 @@ gb_button_get_standard_properties (GtkWidget * widget,
 	    gb_widget_output_choice (data, relief_p, i, GladeReliefSymbols[i]);
 	}
     }
+
+  if (focus_on_click_p)
+    gb_widget_output_bool (data, focus_on_click_p,
+			   gtk_button_get_focus_on_click (GTK_BUTTON (widget)));
 }
 
 
@@ -1705,7 +1247,7 @@ gb_button_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
   gboolean response_id_visible = FALSE;
 
   gb_button_get_standard_properties (widget, data, StockButton, Label, Icon,
-				     Relief);
+				     Relief, FocusOnClick);
 
   /* Handle the response id for dialog buttons here. */
   button_type = gb_button_get_button_type (widget);
@@ -1739,7 +1281,8 @@ gb_button_set_standard_properties	(GtkWidget * widget,
 					 gchar *stock_id_p,
 					 gchar *label_p,
 					 gchar *icon_p,
-					 gchar *relief_p)
+					 gchar *relief_p,
+					 gchar *focus_on_click_p)
 {
   GladeButtonType button_type;
   gchar *relief;
@@ -1753,10 +1296,6 @@ gb_button_set_standard_properties	(GtkWidget * widget,
       gb_button_normal_set_properties (widget, data, stock_id_p, label_p,
 				       icon_p);
       break;
-    case GLADE_BUTTON_TOOLBAR:
-      gb_button_toolbar_set_properties (widget, data, stock_id_p, label_p,
-					icon_p);
-      break;
     case GLADE_BUTTON_DIALOG:
       gb_button_normal_set_properties (widget, data, stock_id_p, label_p,
 				       icon_p);
@@ -1768,23 +1307,29 @@ gb_button_set_standard_properties	(GtkWidget * widget,
     }
 
   /* Handle the Relief property here, as it is simple. */
-  if (button_type != GLADE_BUTTON_TOOLBAR)
+  relief = gb_widget_input_choice (data, relief_p);
+  if (data->apply)
     {
-      relief = gb_widget_input_choice (data, relief_p);
-      if (data->apply)
+      for (i = 0; i < GladeReliefChoicesSize; i++)
 	{
-	  for (i = 0; i < GladeReliefChoicesSize; i++)
+	  if (!strcmp (relief, GladeReliefChoices[i])
+	      || !strcmp (relief, GladeReliefSymbols[i]))
 	    {
-	      if (!strcmp (relief, GladeReliefChoices[i])
-		  || !strcmp (relief, GladeReliefSymbols[i]))
-		{
-		  gtk_button_set_relief (GTK_BUTTON (widget),
-					 GladeReliefValues[i]);
-		  break;
-		}
+	      gtk_button_set_relief (GTK_BUTTON (widget),
+				     GladeReliefValues[i]);
+	      break;
 	    }
 	}
     }
+
+  if (focus_on_click_p)
+    {
+      gboolean focus_on_click;
+
+      focus_on_click = gb_widget_input_bool (data, focus_on_click_p);
+      if (data->apply)
+	gtk_button_set_focus_on_click (GTK_BUTTON (widget), focus_on_click);
+    }
 }
 
 
@@ -1804,7 +1349,7 @@ gb_button_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 				      GladeButtonStockIDKey);
 
   gb_button_set_standard_properties (widget, data, StockButton, Label, Icon,
-				     Relief);
+				     Relief, FocusOnClick);
 
   /* Handle the response id for dialog buttons here. */
   button_type = gb_button_get_button_type (widget);
@@ -1910,8 +1455,7 @@ gb_button_create_popup_menu (GtkWidget * widget, GbWidgetCreateMenuData * data)
   /* If the button's child isn't a placeholder, add a command to remove the
      button's contents. */
   child = GTK_BIN (widget)->child;
-  if (child && !GB_IS_PLACEHOLDER (child)
-      && !gb_toolbar_is_toolbar_button (widget))
+  if (child && !GB_IS_PLACEHOLDER (child))
     {
       GtkWidget *menuitem;
 
@@ -1921,9 +1465,6 @@ gb_button_create_popup_menu (GtkWidget * widget, GbWidgetCreateMenuData * data)
       gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
 			  GTK_SIGNAL_FUNC (gb_button_remove_contents), widget);
     }
-
-  /* Add commands for toolbar buttons. */
-  gb_toolbar_create_toolbar_button_popup_menu (widget, data);
 }
 
 
@@ -1944,10 +1485,6 @@ gb_button_write_standard_source (GtkWidget * widget,
 	case GLADE_BUTTON_NORMAL:
 	  gb_button_normal_write_source (widget, data);
 	  break;
-	case GLADE_BUTTON_TOOLBAR:
-	  gb_button_toolbar_write_source (widget, data);
-	  set_relief = FALSE;
-	  break;
 	case GLADE_BUTTON_DIALOG:
 	  gb_button_normal_write_source (widget, data);
 	  break;
@@ -1969,6 +1506,13 @@ gb_button_write_standard_source (GtkWidget * widget,
 			data->wname, GladeReliefSymbols[i]);
 	}
     }
+
+  if (!gtk_button_get_focus_on_click (GTK_BUTTON (widget)))
+    {
+      source_add (data,
+		  "  gtk_button_set_focus_on_click (GTK_BUTTON (%s), FALSE);\n",
+		  data->wname);
+    }
 }
 
 
