@@ -52,6 +52,13 @@ static gchar *Resizable = "Dialog|GtkWindow::resizable";
 static gchar *DestroyWithParent = "Dialog|GtkWindow::destroy_with_parent";
 static gchar *Icon = "Dialog|GtkWindow::icon";
 
+static gchar *Role = "Dialog|GtkWindow::role";
+static gchar *TypeHint = "Dialog|GtkWindow::type_hint";
+static gchar *SkipTaskbar = "Dialog|GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "Dialog|GtkWindow::skip_pager_hint";
+static gchar *Decorated = "Dialog|GtkWindow::decorated";
+static gchar *Gravity = "Dialog|GtkWindow::gravity";
+
 static gchar *HasSeparator = "GtkDialog::has_separator";
 
 
@@ -381,7 +388,9 @@ gb_dialog_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, AutoShrink,
 					WMName, WMClass,
-					Resizable, DestroyWithParent, Icon);
+					Resizable, DestroyWithParent, Icon,
+					Role, TypeHint, SkipTaskbar,
+					SkipPager, Decorated, Gravity);
   property_add_bool (HasSeparator, _("Has Separator:"),
 		     _("If the dialog has a horizontal separator above the buttons"));
 }
@@ -400,7 +409,9 @@ gb_dialog_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
 				     WMName, WMClass,
-				     Resizable, DestroyWithParent, Icon);
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
   gb_widget_output_bool (data, HasSeparator,
 			 gtk_dialog_get_has_separator (GTK_DIALOG (widget)));
 }
@@ -421,7 +432,9 @@ gb_dialog_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
 				     WMName, WMClass,
-				     Resizable, DestroyWithParent, Icon);
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
   has_separator = gb_widget_input_bool (data, HasSeparator);
   if (data->apply)
     {
@@ -468,7 +481,9 @@ gb_dialog_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
 				   WMName, WMClass,
-				   Resizable, DestroyWithParent, Icon);
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
 
   if (!gtk_dialog_get_has_separator (GTK_DIALOG (widget)))
     {
