@@ -46,6 +46,13 @@ static gchar *Resizable = "FileSel|GtkWindow::resizable";
 static gchar *DestroyWithParent = "FileSel|GtkWindow::destroy_with_parent";
 static gchar *Icon = "FileSel|GtkWindow::icon";
 
+static gchar *Role = "FileSel|GtkWindow::role";
+static gchar *TypeHint = "FileSel|GtkWindow::type_hint";
+static gchar *SkipTaskbar = "FileSel|GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "FileSel|GtkWindow::skip_pager_hint";
+static gchar *Decorated = "FileSel|GtkWindow::decorated";
+static gchar *Gravity = "FileSel|GtkWindow::gravity";
+
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
  * to the function in the GbWidget struct at the end of this file.
@@ -97,7 +104,9 @@ gb_file_selection_create_properties (GtkWidget * widget,
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, AutoShrink,
 					WMName, WMClass,
-					Resizable, DestroyWithParent, Icon);
+					Resizable, DestroyWithParent, Icon,
+					Role, TypeHint, SkipTaskbar,
+					SkipPager, Decorated, Gravity);
   property_add_bool (FileOps, _("File Ops.:"),
 		     _("If the file operation buttons are shown"));
 }
@@ -117,7 +126,9 @@ gb_file_selection_get_properties (GtkWidget * widget,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
 				     WMName, WMClass,
-				     Resizable, DestroyWithParent, Icon);
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
   gb_widget_output_bool (data, FileOps,
 			 GTK_FILE_SELECTION (widget)->fileop_c_dir != NULL);
 }
@@ -139,7 +150,9 @@ gb_file_selection_set_properties (GtkWidget * widget,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
 				     WMName, WMClass,
-				     Resizable, DestroyWithParent, Icon);
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 
   show_fileops = gb_widget_input_bool (data, FileOps);
   if (data->apply)
@@ -195,7 +208,9 @@ gb_file_selection_write_source (GtkWidget * widget,
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
 				   WMName, WMClass,
-				   Resizable, DestroyWithParent, Icon);
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
 
   if (GTK_FILE_SELECTION (widget)->fileop_c_dir == NULL)
     {
