@@ -18,6 +18,18 @@
 
 #include <config.h>
 
+/* This is just for testing, to make sure we don't use deprecated calls. */
+#if 0
+#include <gdk/gdk.h>
+#include <gtk/gtkcontainer.h>
+#include <gtk/gtkenums.h>
+#include <gtk/gtktooltips.h>
+#include <gtk/gtktoolitem.h>
+#define GTK_DISABLE_DEPRECATED
+#include <gtk/gtktoolbar.h>
+#undef GTK_DISABLE_DEPRECATED
+#endif
+
 #ifdef USE_GNOME
 #include <gnome.h>
 #include "../glade_gnome.h"
@@ -32,6 +44,7 @@
 #include <gtk/gtkradiobutton.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtktoolbar.h>
+#include <gtk/gtktoolbutton.h>
 #include <gtk/gtkvbox.h>
 #endif
 
@@ -50,9 +63,11 @@ static const gchar *Size = "GtkToolbar::size";
 static const gchar *Orientation = "GtkToolbar::orientation";
 static const gchar *Style = "GtkToolbar::toolbar_style";
 static const gchar *Tooltips = "GtkToolbar::tooltips";
+static const gchar *ShowArrow = "GtkToolbar::show_arrow";
 
 /* For children of a toolbar */
-static const gchar *GbNewToolbarGroup = "GtkToolbarChild::new_group";
+static const gchar *Expand = "GtkToolbarChild::expand";
+static const gchar *Homogeneous = "GtkToolbarChild::homogeneous";
 
 
 static const gchar *GbOrientationChoices[] =
@@ -101,11 +116,6 @@ static void gb_toolbar_insert_after (GtkWidget * menuitem,
 static void gb_toolbar_insert (GtkWidget * child,
 			       gint offset);
 
-static void gb_toolbar_set_new_toolbar_group	(GtkWidget * toolbar,
-						 GtkWidget * widget,
-						 gboolean new_group);
-static void gb_toolbar_convert_toolbar_button (GtkWidget * menuitem,
-					       GtkWidget * widget);
 
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
@@ -140,27 +150,20 @@ gb_toolbar_new(GbWidgetNewData *data)
 void
 gb_toolbar_add_child (GtkWidget *widget, GtkWidget *child, GbWidgetSetArgData *data)
 {
-  gchar *class_name = NULL;
-  gboolean new_group;
-
-  /* Tell the load functions to use the child properties array. */
-  data->loading_type = GB_CHILD_PROPERTIES;
-  new_group = load_bool (data, "new_group");
-  data->loading_type = GB_STANDARD_PROPERTIES;
-
-  if (new_group)
-    gtk_toolbar_append_space (GTK_TOOLBAR (widget));
-
-  /* See if this is a special toolbar button. */
-  if (data->widget_info)
-    class_name = data->widget_info->class;
-
-  if (class_name && (!strcmp (class_name, "button")
-		     || !strcmp (class_name, "toggle")
-		     || !strcmp (class_name, "radio")))
-    gb_toolbar_insert_toolbar_child (GTK_TOOLBAR (widget), child, -1);
+  /* If the child is a placeholder, we need to insert a GtkToolItem. */
+  if (GB_IS_PLACEHOLDER (child))
+    {
+      GtkWidget *toolitem = (GtkWidget*) gtk_tool_item_new ();
+      gtk_widget_show (toolitem);
+      gtk_container_add (GTK_CONTAINER (toolitem), child);
+      gtk_toolbar_insert (GTK_TOOLBAR (widget),
+			  GTK_TOOL_ITEM (toolitem), -1);
+    }
   else
-    gtk_toolbar_append_widget (GTK_TOOLBAR (widget), child, NULL, NULL);
+    {
+      gtk_toolbar_insert (GTK_TOOLBAR (widget),
+			  GTK_TOOL_ITEM (child), -1);
+    }
 }
 
 
@@ -201,7 +204,7 @@ show_toolbar_dialog (GbWidgetNewData * data)
 static void
 on_toolbar_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
 {
-  GtkWidget *new_widget, *spinbutton, *window, *placeholder;
+  GtkWidget *new_widget, *spinbutton, *window, *toolitem, *placeholder;
   gint items, i;
 
   window = gtk_widget_get_toplevel (widget);
@@ -216,9 +219,13 @@ on_toolbar_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
       new_widget = gtk_toolbar_new ();
       for (i = 0; i < items; i++)
 	{
+	  toolitem = (GtkWidget*) gtk_tool_item_new ();
+	  gtk_widget_show (toolitem);
+	  gtk_toolbar_insert (GTK_TOOLBAR (new_widget),
+			      GTK_TOOL_ITEM (toolitem), -1);
+
 	  placeholder = editor_new_placeholder ();
-	  gtk_toolbar_append_widget (GTK_TOOLBAR (new_widget), placeholder,
-				     NULL, NULL);
+	  gtk_container_add (GTK_CONTAINER (toolitem), placeholder);
 	}
       gb_widget_initialize (new_widget, data);
       (*data->callback) (new_widget, data);
@@ -252,6 +259,7 @@ gb_toolbar_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
 		       _("The toolbar style"),
 		       GbStyleChoices);
   property_add_bool (Tooltips, _("Tooltips:"), _("If tooltips are enabled"));
+  property_add_bool (ShowArrow, _("Show Arrow:"), _("If an arrow should be shown to popup a menu if the toolbar doesn't fit"));
 }
 
 
@@ -266,7 +274,8 @@ gb_toolbar_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
   gint i;
 
   if (data->action != GB_SAVING)
-    gb_widget_output_int (data, Size, GTK_TOOLBAR (widget)->num_children);
+    gb_widget_output_int (data, Size,
+			  gtk_toolbar_get_n_items (GTK_TOOLBAR (widget)));
 
   for (i = 0; i < sizeof (GbOrientationValues)
 	 / sizeof (GbOrientationValues[0]); i++)
@@ -283,6 +292,9 @@ gb_toolbar_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 
   gb_widget_output_bool (data, Tooltips,
 		    GTK_TOOLTIPS (GTK_TOOLBAR (widget)->tooltips)->enabled);
+
+  gb_widget_output_bool (data, ShowArrow,
+			 gtk_toolbar_get_show_arrow (GTK_TOOLBAR (widget)));
 }
 
 
@@ -296,7 +308,7 @@ gb_toolbar_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 {
   gchar *orientation, *style;
   gint i, size;
-  gboolean tooltips;
+  gboolean tooltips, show_arrow;
 
   size = gb_widget_input_int (data, Size);
   if (data->apply)
@@ -340,6 +352,10 @@ gb_toolbar_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
   tooltips = gb_widget_input_bool (data, Tooltips);
   if (data->apply)
     gtk_toolbar_set_tooltips (GTK_TOOLBAR (widget), tooltips);
+
+  show_arrow = gb_widget_input_bool (data, ShowArrow);
+  if (data->apply)
+    gtk_toolbar_set_show_arrow (GTK_TOOLBAR (widget), show_arrow);
 }
 
 
@@ -348,8 +364,9 @@ gb_toolbar_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 static void
 update_toolbar_size (GtkWidget * widget, gint size)
 {
-  GtkWidget *placeholder;
-  gint current_size = GTK_TOOLBAR (widget)->num_children;
+  gint current_size;
+
+  current_size = gtk_toolbar_get_n_items (GTK_TOOLBAR (widget));
 
   if (current_size < size)
     {
@@ -358,42 +375,41 @@ update_toolbar_size (GtkWidget * widget, gint size)
 
       while (current_size < size)
 	{
+	  GtkWidget *toolitem, *placeholder;
+
+	  toolitem = (GtkWidget*) gtk_tool_item_new ();
+	  gtk_widget_show (toolitem);
+	  gtk_toolbar_insert (GTK_TOOLBAR (widget),
+			      GTK_TOOL_ITEM (toolitem), -1);
+
 	  placeholder = editor_new_placeholder ();
-	  gtk_toolbar_append_widget (GTK_TOOLBAR (widget), placeholder,
-				     NULL, NULL);
+	  gtk_container_add (GTK_CONTAINER (toolitem), placeholder);
+
 	  current_size++;
 	}
     }
   else if (current_size > size)
     {
-      GList *item = g_list_nth (GTK_TOOLBAR (widget)->children, size);
-      while (item)
+      while (current_size > size)
 	{
-	  GtkToolbarChild *child = (GtkToolbarChild*) item->data;
-	  item = item->next;
-	  if (child->type == GTK_TOOLBAR_CHILD_SPACE)
-	    {
-	      /* Note that we are always removing the element at size. */
-	      gtk_toolbar_remove_space (GTK_TOOLBAR (widget), size);
-	    }
-	  else
-	    {
-	      gtk_container_remove (GTK_CONTAINER (widget), child->widget);
-	    }
+	  GtkToolItem *item;
+
+	  item = gtk_toolbar_get_nth_item (GTK_TOOLBAR (widget), size);
+	  gtk_container_remove (GTK_CONTAINER (widget), GTK_WIDGET (item));
+	  current_size--;
 	}
     }
 }
 
 
-/*
- * Creates the child packing properties for children of this widget.
- */
 static void
 gb_toolbar_create_child_properties (GtkWidget * widget,
 				    GbWidgetCreateChildArgData * data)
 {
-  property_add_bool (GbNewToolbarGroup, _("New Group:"),
-		     _("Set True to start a new group of toolbar items"));
+  property_add_bool (Expand, _("Expand:"),
+		     _("Set True to let the widget expand"));
+  property_add_bool (Homogeneous, _("Homogeneous:"),
+		     _("If the item should be the same size as other homogeneous items"));
 }
 
 
@@ -401,33 +417,37 @@ static void
 gb_toolbar_get_child_properties (GtkWidget *widget, GtkWidget *child,
 				 GbWidgetGetArgData *data)
 {
-  GtkToolbarChild *tchild;
-  gint pos = 0;
+  if (data->action == GB_SAVING)
+    save_start_tag (data, "packing");
 
-  tchild = glade_util_find_toolbar_child (widget, child, &pos, NULL);
-  if (pos == 0)
-    {
-      if (data->action == GB_SHOWING)
-	{
-	  gb_widget_output_bool (data, GbNewToolbarGroup, FALSE);
-	  property_set_sensitive (GbNewToolbarGroup, FALSE);
-	}
-    }
-  else
-    {
-      gboolean new_group = gb_toolbar_get_new_toolbar_group (widget, child);
-      if (data->action == GB_SHOWING)
-	{
-	  property_set_sensitive (GbNewToolbarGroup, TRUE);
-	  gb_widget_output_bool (data, GbNewToolbarGroup, new_group);
-	}
-      else if (new_group && data->action == GB_SAVING)
-	{
-	  save_start_tag (data, "packing");
-	  gb_widget_output_bool (data, GbNewToolbarGroup, new_group);
-	  save_end_tag (data, "packing");
-	}
-    }
+  gb_widget_output_bool (data, Expand,
+			 gtk_tool_item_get_expand (GTK_TOOL_ITEM (child)));
+  gb_widget_output_bool (data, Homogeneous,
+			 gtk_tool_item_get_homogeneous (GTK_TOOL_ITEM (child)));
+
+  if (data->action == GB_SAVING)
+    save_end_tag (data, "packing");
+}
+
+
+/* Migrate old 2.0 XML files. We insert a separator tool item. */
+static void
+gb_toolbar_set_new_group (GtkWidget *widget, GtkWidget *child)
+{
+  gint pos;
+  GtkWidget *separator;
+
+  pos = gtk_toolbar_get_item_index (GTK_TOOLBAR (widget),
+				    GTK_TOOL_ITEM (child));
+
+  /* We use gb_widget_new_full() passing FALSE for create_default_name,
+     since we are loading and we don't want to create any names while loading
+     or they may clash. */
+  separator = gb_widget_new_full ("GtkSeparatorToolItem", FALSE,
+				  widget, NULL, 0, 0, NULL,
+				  GB_LOADING, NULL);
+
+  gtk_toolbar_insert (GTK_TOOLBAR (widget), GTK_TOOL_ITEM (separator), pos);
 }
 
 
@@ -435,9 +455,23 @@ static void
 gb_toolbar_set_child_properties (GtkWidget *widget, GtkWidget *child,
 				 GbWidgetSetArgData *data)
 {
-  gboolean new_group = gb_widget_input_bool (data, GbNewToolbarGroup);
+  gboolean new_group, expand, homogeneous;
+
+  /* Migrate old 2.0 XML files. */
+  if (data->action == GB_LOADING)
+    {
+      new_group = gb_widget_input_bool (data, "new_group");
+      if (data->apply && new_group)
+	gb_toolbar_set_new_group (widget, child);
+    }
+
+  expand = gb_widget_input_bool (data, Expand);
   if (data->apply)
-    gb_toolbar_set_new_toolbar_group (widget, child, new_group);
+    gtk_tool_item_set_expand (GTK_TOOL_ITEM (child), expand);
+
+  homogeneous = gb_widget_input_bool (data, Homogeneous);
+  if (data->apply)
+    gtk_tool_item_set_homogeneous (GTK_TOOL_ITEM (child), homogeneous);
 }
 
 /*
@@ -449,32 +483,24 @@ static void
 gb_toolbar_create_popup_menu(GtkWidget *widget, GbWidgetCreateMenuData *data)
 {
   GtkWidget *menuitem;
-  GList *child_element;
-  GtkToolbarChild *child;
 
   if (data->child == NULL)
     return;
 
-  /* Find the toolbar child corresponding to the button the mouse is over. */
-  child = glade_util_find_toolbar_child (widget, data->child, NULL, &child_element);
-
   /* Commands for inserting new items. */
-  if (child)
-    {
-      menuitem = gtk_menu_item_new_with_label (_("Insert Item Before"));
-      gtk_widget_show (menuitem);
-      gtk_container_add (GTK_CONTAINER (data->menu), menuitem);
-      gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-			  GTK_SIGNAL_FUNC (gb_toolbar_insert_before),
-			  data->child);
+  menuitem = gtk_menu_item_new_with_label (_("Insert Item Before"));
+  gtk_widget_show (menuitem);
+  gtk_container_add (GTK_CONTAINER (data->menu), menuitem);
+  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
+		      GTK_SIGNAL_FUNC (gb_toolbar_insert_before),
+		      data->child);
 
-      menuitem = gtk_menu_item_new_with_label (_("Insert Item After"));
-      gtk_widget_show (menuitem);
-      gtk_container_add (GTK_CONTAINER (data->menu), menuitem);
-      gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-			  GTK_SIGNAL_FUNC (gb_toolbar_insert_after),
-			  data->child);
-    }
+  menuitem = gtk_menu_item_new_with_label (_("Insert Item After"));
+  gtk_widget_show (menuitem);
+  gtk_container_add (GTK_CONTAINER (data->menu), menuitem);
+  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
+		      GTK_SIGNAL_FUNC (gb_toolbar_insert_after),
+		      data->child);
 }
 
 
@@ -495,272 +521,25 @@ gb_toolbar_insert_after (GtkWidget * menuitem, GtkWidget * child)
 static void
 gb_toolbar_insert (GtkWidget * child, gint offset)
 {
-  GtkWidget *toolbar = child->parent, *placeholder;
-  GtkToolbarChild *tbchild;
+  GtkWidget *toolbar = child->parent, *toolitem, *placeholder;
   gint pos;
 
-  tbchild = glade_util_find_toolbar_child (toolbar, child, &pos, NULL);
-  g_return_if_fail (tbchild != NULL);
+  pos = gtk_toolbar_get_item_index (GTK_TOOLBAR (toolbar),
+				    GTK_TOOL_ITEM (child));
 
   if (offset > 0)
     pos++;
 
+  toolitem = (GtkWidget*) gtk_tool_item_new ();
+  gtk_widget_show (toolitem);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), (GtkToolItem*) toolitem, pos);
+
   placeholder = editor_new_placeholder ();
-  gtk_toolbar_insert_widget (GTK_TOOLBAR (toolbar), placeholder,
-			     NULL, NULL, pos);
+  gtk_container_add (GTK_CONTAINER (toolitem), placeholder);
 }
 
 
 
-
-
-/*************************************************************************
- * Toolbar helper functions.
- *************************************************************************/
-
-/* This returns TRUE if the widget is a special toolbar button, i.e. it
-   has an icon and label. */
-gboolean
-gb_toolbar_is_toolbar_button (GtkWidget *widget)
-{
-  gchar *child_name = gb_widget_get_child_name (widget);
-  return (child_name && !strcmp (child_name, GladeChildToolbarButton));
-}
-
-
-gboolean
-gb_toolbar_get_new_toolbar_group	(GtkWidget	       *toolbar,
-				 GtkWidget	       *widget)
-{
-  GtkToolbarChild *tbchild, *prev_tbchild;
-  GList *elem;
-
-  tbchild = glade_util_find_toolbar_child (toolbar, widget, NULL, &elem);
-  g_return_val_if_fail (tbchild != NULL, FALSE);
-
-  if (elem->prev == NULL)
-    return FALSE;
-
-  prev_tbchild = (GtkToolbarChild*) elem->prev->data;
-  if (prev_tbchild->type == GTK_TOOLBAR_CHILD_SPACE)
-    return TRUE;
-
-  return FALSE;
-}
-
-
-/* This sets whether or not a toolbar child starts a new group on the toolbar,
-   i.e. there is a space before it. */
-static void
-gb_toolbar_set_new_toolbar_group (GtkWidget	       *toolbar,
-				 GtkWidget	       *widget,
-				 gboolean		new_group)
-{
-  GtkToolbarChild *tbchild, *prev_tbchild;
-  gint pos;
-  GList *elem;
-
-  tbchild = glade_util_find_toolbar_child (toolbar, widget, &pos, &elem);
-  g_return_if_fail (tbchild != NULL);
-
-  if (elem->prev == NULL)
-    return;
-
-  prev_tbchild = (GtkToolbarChild*) elem->prev->data;
-
-  if (new_group)
-    {
-      if (prev_tbchild->type != GTK_TOOLBAR_CHILD_SPACE)
-	{
-	  gtk_toolbar_insert_space (GTK_TOOLBAR (toolbar), pos);
-	}
-    }
-  else
-    {
-      if (prev_tbchild->type == GTK_TOOLBAR_CHILD_SPACE)
-	{
-	  gtk_toolbar_remove_space (GTK_TOOLBAR (toolbar), pos - 1);
-	}
-    }
-}
-
-
-void
-gb_toolbar_create_toolbar_button_popup_menu (GtkWidget	           *widget,
-					     GbWidgetCreateMenuData *data)
-{
-  GtkWidget *menuitem;
-
-  if (gb_toolbar_is_toolbar_button (widget))
-    {
-      menuitem = gtk_menu_item_new_with_label (_("Convert to Ordinary Button"));
-      gtk_widget_show (menuitem);
-      gtk_container_add (GTK_CONTAINER (data->menu), menuitem);
-      gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-			  GTK_SIGNAL_FUNC (gb_toolbar_convert_toolbar_button),
-			  widget);
-    }
-}
-
-
-static void
-gb_toolbar_convert_toolbar_button (GtkWidget * menuitem,
-				   GtkWidget * widget)
-{
-  GtkWidget *button;
-  GtkToolbarChild *child;
-
-  g_return_if_fail (widget->parent != NULL);
-  child = glade_util_find_toolbar_child (widget->parent, widget, NULL, NULL);
-  g_return_if_fail (child != NULL);
-
-  /* We set the parent to NULL when calling gb_widget_new, otherwise it
-     creates a new toolbar button! */
-  if (child->type == GTK_TOOLBAR_CHILD_BUTTON)
-    button = gb_widget_new ("GtkButton", NULL);
-  else if (child->type == GTK_TOOLBAR_CHILD_TOGGLEBUTTON)
-    button = gb_widget_new ("GtkToggleButton", NULL);
-  else if (child->type == GTK_TOOLBAR_CHILD_RADIOBUTTON)
-    button = gb_widget_new ("GtkRadioButton", NULL);
-  else
-    {
-      g_warning ("Invalid toolbar button");
-      return;
-    }
-
-  /* Set properties widget to NULL, in case the widget or parent is replaced */
-  property_set_widget (NULL);
-
-  gb_widget_replace_child (widget->parent, widget, button);
-}
-
-
-GtkWidget *
-gb_toolbar_new_toolbar_button (GbWidgetNewData *data,
-			       GtkToolbarChildType type)
-{
-  GtkToolbar *toolbar = GTK_TOOLBAR (data->parent);
-  GtkWidget *new_widget, *box, *icon, *label;
-  GbWidget * pixmap_gbwidget;
-
-  /* If we are loading, we check that a toolbar button is needed. */
-  MSG ("In gb_widget_new_toolbar_button");
-  if (data->action == GB_LOADING)
-    {
-      gchar *class_name = NULL;
-
-      if (data->loading_data->widget_info)
-	class_name = data->loading_data->widget_info->class;
-
-      if (!class_name)
-	return NULL;
-
-      if (strcmp (class_name, "button")
-	  && strcmp (class_name, "toggle")
-	  && strcmp (class_name, "radio"))
-	return NULL;
-    }
-
-  if (type == GTK_TOOLBAR_CHILD_BUTTON)
-    {
-      new_widget = gtk_button_new ();
-    }
-  else if (type == GTK_TOOLBAR_CHILD_TOGGLEBUTTON)
-    {
-      new_widget = gtk_toggle_button_new ();
-      gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (new_widget), FALSE);
-    }
-  else
-    {
-      new_widget = gtk_radio_button_new (NULL);
-      gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (new_widget), FALSE);
-    }
-
-  /* Remember it is a special widget. */
-  gb_widget_set_child_name (new_widget, GladeChildToolbarButton);
-
-  GTK_WIDGET_UNSET_FLAGS (new_widget, GTK_CAN_FOCUS);
-  if (toolbar->style == GTK_TOOLBAR_BOTH_HORIZ)
-    box = gtk_hbox_new (FALSE, 0);
-  else
-    box = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (new_widget), box);
-  gtk_widget_show (box);
-
-  label = gtk_label_new (data->name);
-  gtk_box_pack_end (GTK_BOX (box), label, FALSE, FALSE, 0);
-  if (toolbar->style != GTK_TOOLBAR_ICONS)
-    gtk_widget_show (label);
-
-  pixmap_gbwidget = gb_widget_lookup_class ("GtkImage");
-  if (pixmap_gbwidget)
-    {
-      icon = gtk_image_new_from_pixmap (pixmap_gbwidget->gdkpixmap,
-					pixmap_gbwidget->mask);
-      gtk_box_pack_end (GTK_BOX (box), icon, FALSE, FALSE, 0);
-      if (toolbar->style != GTK_TOOLBAR_TEXT)
-	gtk_widget_show (icon);
-    }
-  else
-    g_warning ("Couldn't find GtkPixmap data");
-
-  return new_widget;
-}
-
-
-/* SPECIAL CODE: This inserts a proper toolbar button (with an icon and label)
-   into a toolbar at the given position. We need it since GtkToolbar creates
-   and inserts the widget all in one go, which isn't good enough for us,
-   as we want to set the events mask to add signal handlers to the buttons.
-   This also sets the radio button group, if it is a radio button. */
-void
-gb_toolbar_insert_toolbar_child (GtkToolbar *toolbar,
-				 GtkWidget *new_child,
-				 gint position)
-{
-  GtkToolbarChild *child;
-  GtkWidget *icon, *label;
-  GtkToolbarChildType type;
-
-  MSGIN;
-
-  /* Determine if the new child is a proper toolbar button, i.e. it has
-     a vbox with a pixmap and a label in it. If it isn't we just add it as
-     a standard widget. */
-  if (!gb_toolbar_is_toolbar_button (new_child))
-    {
-      gtk_toolbar_insert_widget (toolbar, new_child, NULL, NULL, position);
-      return;
-    }
-
-  if (!gb_button_toolbar_find_child_widgets (new_child, &label, &icon))
-    return;
-  if (GTK_IS_RADIO_BUTTON (new_child))
-    type = GTK_TOOLBAR_CHILD_RADIOBUTTON;
-  else if (GTK_IS_TOGGLE_BUTTON (new_child))
-    type = GTK_TOOLBAR_CHILD_TOGGLEBUTTON;
-  else
-    type = GTK_TOOLBAR_CHILD_BUTTON;
-
-  child = g_new (GtkToolbarChild, 1);
-  child->widget = new_child;
-  child->icon = icon;
-  child->label = label;
-  child->type = type;
-
-  /* This is mainly copied from gtktoolbar.c */
-  toolbar->children = g_list_insert (toolbar->children, child, position);
-  toolbar->num_children++;
-
-  gtk_widget_set_parent (child->widget, GTK_WIDGET (toolbar));
-}
-
-
-/*************************************************************************
- * END Toolbar helper functions.
- *************************************************************************/
-
-
 /*
  * Writes the source code needed to create this widget.
  * You have to output everything necessary to create the widget here, though
@@ -809,30 +588,60 @@ gb_toolbar_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
     source_add (data,
 		"  gtk_toolbar_set_tooltips (GTK_TOOLBAR (%s), FALSE);\n",
 		data->wname);
+
+  if (!gtk_toolbar_get_show_arrow (GTK_TOOLBAR (widget)))
+    source_add (data,
+		"  gtk_toolbar_set_show_arrow (GTK_TOOLBAR (%s), FALSE);\n",
+		data->wname);
+
+  /* We set up a tmp_toolbar_icon_size variable that we can use when creating
+     the tool items. */
+  source_ensure_decl (data, "  gint tmp_toolbar_icon_size;\n");
+  source_add (data,
+	      "  tmp_toolbar_icon_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (%s));\n",
+	      data->wname);
 }
 
 
-/* Outputs source to add a child widget to a hbox/vbox. */
 static void
-gb_toolbar_write_add_child_source (GtkWidget * parent,
-				   const gchar *parent_name,
-				   GtkWidget *child,
-				   GbWidgetWriteSourceData * data)
+gb_toolbar_write_add_child_source (GtkWidget               *parent,
+				   const char              *parent_name,
+				   GtkWidget               *child,
+				   GbWidgetWriteSourceData *data)
 {
-  /* For toolbar buttons, the widgets are added to the toolbar when
-     created, so we skip this. For standard widgets, we add them here. */
-  if (!gb_toolbar_is_toolbar_button (child))
-    {
-      if (gb_toolbar_get_new_toolbar_group (parent, child))
-	source_add (data,
-		    "  gtk_toolbar_append_space (GTK_TOOLBAR (%s));\n\n",
-		    parent_name);
+  gboolean homogeneous, output_homogeneous = FALSE;
 
+  if (gtk_tool_item_get_expand (GTK_TOOL_ITEM (child)))
+    {
       source_add (data,
-		  "  gtk_toolbar_append_widget (GTK_TOOLBAR (%s), %s, %s, NULL);\n",
-		  parent_name, data->wname,
-		  data->widget_data->tooltip ? source_make_string (data->widget_data->tooltip, data->use_gettext) : "NULL");
+		  "  gtk_tool_item_set_expand (GTK_TOOL_ITEM (%s), TRUE);\n",
+		  data->wname);
     }
+
+  /* Homogeneous defaults to FALSE for GtkToolItem, but TRUE for
+     GtkToolButton and descendants. */
+  homogeneous = gtk_tool_item_get_homogeneous (GTK_TOOL_ITEM (child));
+  if (GTK_IS_TOOL_BUTTON (child))
+    {
+      if (!homogeneous)
+	output_homogeneous = TRUE;
+    }
+  else
+    {
+      if (homogeneous)
+	output_homogeneous = TRUE;
+    }
+
+  if (output_homogeneous)
+    {
+      source_add (data,
+		  "  gtk_tool_item_set_homogeneous (GTK_TOOL_ITEM (%s), %s);\n",
+		  data->wname,
+		  homogeneous ? "TRUE" : "FALSE");
+    }
+
+  source_add (data, "  gtk_container_add (GTK_CONTAINER (%s), %s);\n",
+	      parent_name, data->wname);
 }
 
 
