@@ -52,6 +52,13 @@ static gchar *Resizable = "GtkWindow::resizable";
 static gchar *DestroyWithParent = "GtkWindow::destroy_with_parent";
 static gchar *Icon = "GtkWindow::icon";
 
+static gchar *Role = "GtkWindow::role";
+static gchar *TypeHint = "GtkWindow::type_hint";
+static gchar *SkipTaskbar = "GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "GtkWindow::skip_pager_hint";
+static gchar *Decorated = "GtkWindow::decorated";
+static gchar *Gravity = "GtkWindow::gravity";
+
 /* These are deprecated. We don't show them any more. */
 static gchar *Shrink = "GtkWindow::allow_shrink";
 static gchar *Grow = "GtkWindow::allow_grow";
@@ -73,19 +80,79 @@ static const gchar *GbTypeSymbols[] =
   "GTK_WINDOW_POPUP"
 };
 
+static const gchar *GbTypeHintChoices[] =
+{"Normal", "Dialog", "Menu", "Toolbar", "Splash Screen", "Utility",
+ "Dock", "Desktop", NULL};
+static const gint GbTypeHintValues[] =
+{
+  GDK_WINDOW_TYPE_HINT_NORMAL,
+  GDK_WINDOW_TYPE_HINT_DIALOG,
+  GDK_WINDOW_TYPE_HINT_MENU,
+  GDK_WINDOW_TYPE_HINT_TOOLBAR,
+  GDK_WINDOW_TYPE_HINT_SPLASHSCREEN,
+  GDK_WINDOW_TYPE_HINT_UTILITY,
+  GDK_WINDOW_TYPE_HINT_DOCK,
+  GDK_WINDOW_TYPE_HINT_DESKTOP
+};
+static const gchar *GbTypeHintSymbols[] =
+{
+  "GDK_WINDOW_TYPE_HINT_NORMAL",
+  "GDK_WINDOW_TYPE_HINT_DIALOG",
+  "GDK_WINDOW_TYPE_HINT_MENU",
+  "GDK_WINDOW_TYPE_HINT_TOOLBAR",
+  "GDK_WINDOW_TYPE_HINT_SPLASHSCREEN",
+  "GDK_WINDOW_TYPE_HINT_UTILITY",
+  "GDK_WINDOW_TYPE_HINT_DOCK",
+  "GDK_WINDOW_TYPE_HINT_DESKTOP"
+};
+
 static const gchar *GbPositionChoices[] =
-{"None", "Center", "Mouse", NULL};
+{"None", "Center", "Mouse", "Center Always", "Center on Parent", NULL};
 static const gint GbPositionValues[] =
 {
   GTK_WIN_POS_NONE,
   GTK_WIN_POS_CENTER,
-  GTK_WIN_POS_MOUSE
+  GTK_WIN_POS_MOUSE,
+  GTK_WIN_POS_CENTER_ALWAYS,
+  GTK_WIN_POS_CENTER_ON_PARENT
 };
 static const gchar *GbPositionSymbols[] =
 {
   "GTK_WIN_POS_NONE",
   "GTK_WIN_POS_CENTER",
-  "GTK_WIN_POS_MOUSE"
+  "GTK_WIN_POS_MOUSE",
+  "GTK_WIN_POS_CENTER_ALWAYS",
+  "GTK_WIN_POS_CENTER_ON_PARENT"
+};
+
+static const gchar *GbGravityChoices[] =
+{"North West", "North", "North East", "West", "Center", "East",
+ "South West", "South", "South East", "Static", NULL};
+static const gint GbGravityValues[] =
+{
+  GDK_GRAVITY_NORTH_WEST,
+  GDK_GRAVITY_NORTH,
+  GDK_GRAVITY_NORTH_EAST,
+  GDK_GRAVITY_WEST,
+  GDK_GRAVITY_CENTER,
+  GDK_GRAVITY_EAST,
+  GDK_GRAVITY_SOUTH_WEST,
+  GDK_GRAVITY_SOUTH,
+  GDK_GRAVITY_SOUTH_EAST,
+  GDK_GRAVITY_STATIC
+};
+static const gchar *GbGravitySymbols[] =
+{
+  "GDK_GRAVITY_NORTH_WEST",
+  "GDK_GRAVITY_NORTH",
+  "GDK_GRAVITY_NORTH_EAST",
+  "GDK_GRAVITY_WEST",
+  "GDK_GRAVITY_CENTER",
+  "GDK_GRAVITY_EAST",
+  "GDK_GRAVITY_SOUTH_WEST",
+  "GDK_GRAVITY_SOUTH",
+  "GDK_GRAVITY_SOUTH_EAST",
+  "GDK_GRAVITY_STATIC"
 };
 
 
@@ -139,7 +206,9 @@ gb_window_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, NULL,
 					WMName, WMClass,
-					Resizable, DestroyWithParent, Icon);
+					Resizable, DestroyWithParent, Icon,
+					Role, TypeHint, SkipTaskbar,
+					SkipPager, Decorated, Gravity);
 }
 
 
@@ -159,7 +228,13 @@ gb_window_create_standard_properties (GtkWidget * widget,
 				      gchar *wmclass_p,
 				      gchar *resizable_p,
 				      gchar *destroy_with_parent_p,
-				      gchar *icon_p)
+				      gchar *icon_p,
+				      gchar *role_p,
+				      gchar *type_hint_p,
+				      gchar *skip_taskbar_p,
+				      gchar *skip_pager_p,
+				      gchar *decorated_p,
+				      gchar *gravity_p)
 {
   if (title_p)
     property_add_string (title_p, _("Title:"), _("The title of the window"));
@@ -168,6 +243,11 @@ gb_window_create_standard_properties (GtkWidget * widget,
     property_add_choice (type_p, _("Type:"), _("The type of the window"),
 			 GbTypeChoices);
 
+  if (type_hint_p)
+    property_add_choice (type_hint_p, _("Type Hint:"),
+			 _("Tells the window manager how to treat the window"),
+			 GbTypeHintChoices);
+
   if (position_p)
     property_add_choice (position_p, _("Position:"),
 			 _("The initial position of the window"),
@@ -222,6 +302,27 @@ gb_window_create_standard_properties (GtkWidget * widget,
   if (icon_p)
     property_add_filename (icon_p, _("Icon:"),
 			   _("The icon for this window"));
+
+  if (role_p)
+    property_add_string (role_p, _("Role:"), _("A unique identifier for the window to be used when restoring a session"));
+
+  if (decorated_p)
+    property_add_bool (decorated_p, _("Decorated:"),
+		       _("If the window should be decorated by the window manager"));
+
+  if (skip_taskbar_p)
+    property_add_bool (skip_taskbar_p, _("Skip Taskbar:"),
+		       _("If the window should not appear in the task bar"));
+
+  if (skip_pager_p)
+    property_add_bool (skip_pager_p, _("Skip Pager:"),
+		       _("If the window should not appear in the pager"));
+
+  if (gravity_p)
+    property_add_choice (gravity_p, _("Gravity:"),
+			 _("The reference point to use when the window coordinates are set"),
+			 GbGravityChoices);
+
 }
 
 
@@ -305,7 +406,9 @@ gb_window_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, NULL,
 				     WMName, WMClass,
-				     Resizable, DestroyWithParent, Icon);
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 }
 
 
@@ -325,9 +428,15 @@ gb_window_get_standard_properties (GtkWidget * widget,
 				   gchar *wmclass_p,
 				   gchar *resizable_p,
 				   gchar *destroy_with_parent_p,
-				   gchar *icon_p)
+				   gchar *icon_p,
+				   gchar *role_p,
+				   gchar *type_hint_p,
+				   gchar *skip_taskbar_p,
+				   gchar *skip_pager_p,
+				   gchar *decorated_p,
+				   gchar *gravity_p)
 {
-  gint type, position, default_width, default_height;
+  gint type, position, default_width, default_height, type_hint, gravity;
 
   if (title_p)
     gb_widget_output_translatable_string (data, title_p,
@@ -410,6 +519,50 @@ gb_window_get_standard_properties (GtkWidget * widget,
     gb_widget_output_pixmap_filename (data, icon_p,
 				      gtk_object_get_data (GTK_OBJECT (widget),
 							   GladeWindowIconKey));
+
+  if (role_p)
+    gb_widget_output_string (data, role_p,
+			     gtk_window_get_role (GTK_WINDOW (widget)));
+
+  /* We can't set the decorated property as that may make it awkward to
+     edit. The default (i.e. NULL value) is TRUE. */
+  if (decorated_p)
+    gb_widget_output_bool (data, decorated_p,
+			   gtk_object_get_data (GTK_OBJECT (widget), decorated_p)
+			   == NULL ? TRUE : FALSE);
+
+  /* The default (i.e. NULL value) is FALSE. */
+  if (skip_taskbar_p)
+    gb_widget_output_bool (data, skip_taskbar_p,
+			   gtk_object_get_data (GTK_OBJECT (widget), skip_taskbar_p)
+			   != NULL ? TRUE : FALSE);
+
+  /* The default (i.e. NULL value) is FALSE. */
+  if (skip_pager_p)
+    gb_widget_output_bool (data, skip_pager_p,
+			   gtk_object_get_data (GTK_OBJECT (widget), skip_pager_p)
+			   != NULL ? TRUE : FALSE);
+
+  /* We can't set the type hint within Glade as it may make the window
+     awkward to edit. It defaults to 0 which is GDK_WINDOW_TYPE_HINT_NORMAL
+     which is OK. */
+  if (type_hint_p)
+    {
+      type_hint = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
+							type_hint_p));
+      gb_widget_output_choice (data, type_hint_p, type_hint,
+			       GbTypeHintSymbols[type_hint]);
+    }
+
+  /* I'm not sure if we should set the gravity or not.
+     It defaults to 0, which is GDK_GRAVITY_NORTH_WEST which is OK. */
+  if (gravity_p)
+    {
+      gravity = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
+						      gravity_p));
+      gb_widget_output_choice (data, gravity_p, gravity,
+			       GbGravitySymbols[gravity]);
+    }
 }
 
 
@@ -426,7 +579,9 @@ gb_window_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, NULL,
 				     WMName, WMClass,
-				     Resizable, DestroyWithParent, Icon);
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 }
 
 
@@ -446,12 +601,20 @@ gb_window_set_standard_properties (GtkWidget * widget,
 				   gchar *wmclass_p,
 				   gchar *resizable_p,
 				   gchar *destroy_with_parent_p,
-				   gchar *icon_p)
+				   gchar *icon_p,
+				   gchar *role_p,
+				   gchar *type_hint_p,
+				   gchar *skip_taskbar_p,
+				   gchar *skip_pager_p,
+				   gchar *decorated_p,
+				   gchar *gravity_p)
 {
-  gchar *title, *type, *position;
+  gchar *title, *type, *position, *role;
   gint default_width, default_height, i;
   gboolean modal, apply_default_width, apply_default_height;
   gboolean resizable, destroy_with_parent;
+  gboolean decorated, skip_taskbar, skip_pager;
+  gchar *type_hint, *gravity;
 #if 0
   gchar *wmname, *wmclass;
 #endif
@@ -639,6 +802,78 @@ gb_window_set_standard_properties (GtkWidget * widget,
       if (data->action == GB_LOADING)
 	g_free (filename);
     }
+
+
+  if (role_p)
+    {
+      role = gb_widget_input_string (data, role_p);
+      if (data->apply)
+	gtk_window_set_role (GTK_WINDOW (widget), role);
+    }
+
+  if (decorated_p)
+    {
+      decorated = gb_widget_input_bool (data, decorated_p);
+      if (data->apply)
+	gtk_object_set_data (GTK_OBJECT (widget), decorated_p,
+			     decorated ? NULL : "FALSE");
+    }
+
+  if (skip_taskbar_p)
+    {
+      skip_taskbar = gb_widget_input_bool (data, skip_taskbar_p);
+      if (data->apply)
+	gtk_object_set_data (GTK_OBJECT (widget), skip_taskbar_p,
+			     skip_taskbar ? "TRUE" : NULL);
+    }
+
+  if (skip_pager_p)
+    {
+      skip_pager = gb_widget_input_bool (data, skip_pager_p);
+      if (data->apply)
+	gtk_object_set_data (GTK_OBJECT (widget), skip_pager_p,
+			     skip_pager ? "TRUE" : NULL);
+    }
+
+  if (type_hint_p)
+    {
+      type_hint = gb_widget_input_choice (data, type_hint_p);
+      if (data->apply)
+	{
+	  for (i = 0;
+	       i < sizeof (GbTypeHintValues) / sizeof (GbTypeHintValues[0]);
+	       i++)
+	    {
+	      if (!strcmp (type_hint, GbTypeHintChoices[i])
+		  || !strcmp (type_hint, GbTypeHintSymbols[i]))
+		{
+		  gtk_object_set_data (GTK_OBJECT (widget), type_hint_p,
+				       GINT_TO_POINTER (i));
+		  break;
+		}
+	    }
+	}
+    }
+
+  if (gravity_p)
+    {
+      gravity = gb_widget_input_choice (data, gravity_p);
+      if (data->apply)
+	{
+	  for (i = 0;
+	       i < sizeof (GbGravityValues) / sizeof (GbGravityValues[0]);
+	       i++)
+	    {
+	      if (!strcmp (gravity, GbGravityChoices[i])
+		  || !strcmp (gravity, GbGravitySymbols[i]))
+		{
+		  gtk_object_set_data (GTK_OBJECT (widget), gravity_p,
+				       GINT_TO_POINTER (i));
+		  break;
+		}
+	    }
+	}
+    }
 }
 
 
@@ -683,7 +918,9 @@ gb_window_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, NULL,
 				   WMName, WMClass,
-				   Resizable, DestroyWithParent, Icon);
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
 }
 
 
@@ -703,9 +940,15 @@ gb_window_write_standard_source (GtkWidget * widget,
 				 gchar *wmclass_p,
 				 gchar *resizable_p,
 				 gchar *destroy_with_parent_p,
-				 gchar *icon_p)
+				 gchar *icon_p,
+				 gchar *role_p,
+				 gchar *type_hint_p,
+				 gchar *skip_taskbar_p,
+				 gchar *skip_pager_p,
+				 gchar *decorated_p,
+				 gchar *gravity_p)
 {
-  gint type, position, default_width, default_height;
+  gint type, position, default_width, default_height, type_hint, gravity;
 #if 0
   gchar *wmname, *wmclass;
 #endif
@@ -840,6 +1083,67 @@ gb_window_write_standard_source (GtkWidget * widget,
 	  g_free (pixbuf_name);
 	}
     }
+
+  if (role_p)
+    {
+      const gchar *role = gtk_window_get_role (GTK_WINDOW (widget));
+
+      /* The role isn't translated. */
+      if (role && *role)
+	source_add (data, "  gtk_window_set_role (GTK_WINDOW (%s), %s);\n",
+		    data->wname,
+		    source_make_string (role, FALSE));
+    }
+
+  if (decorated_p)
+    {
+      if (gtk_object_get_data (GTK_OBJECT (widget), decorated_p) != NULL)
+	{
+	  source_add (data, "  gtk_window_set_decorated (GTK_WINDOW (%s), FALSE);\n",
+		      data->wname);
+	}
+    }
+
+  if (skip_taskbar_p)
+    {
+      if (gtk_object_get_data (GTK_OBJECT (widget), skip_taskbar_p) != NULL)
+	{
+	  source_add (data, "  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (%s), TRUE);\n",
+		      data->wname);
+	}
+    }
+
+  if (skip_pager_p)
+    {
+      if (gtk_object_get_data (GTK_OBJECT (widget), skip_pager_p) != NULL)
+	{
+	  source_add (data, "  gtk_window_set_skip_pager_hint (GTK_WINDOW (%s), TRUE);\n",
+		      data->wname);
+	}
+    }
+
+  if (type_hint_p)
+    {
+      type_hint = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
+							type_hint_p));
+      if (GbTypeHintValues[type_hint] != GDK_WINDOW_TYPE_HINT_NORMAL)
+	{
+	  source_add (data, "  gtk_window_set_type_hint (GTK_WINDOW (%s), %s);\n",
+		      data->wname, GbTypeHintSymbols[type_hint]);
+	}
+
+    }
+
+  if (gravity_p)
+    {
+      gravity = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
+						      gravity_p));
+      if (GbGravityValues[gravity] != GDK_GRAVITY_NORTH_WEST)
+	{
+	  source_add (data, "  gtk_window_set_gravity (GTK_WINDOW (%s), %s);\n",
+		      data->wname, GbGravitySymbols[gravity]);
+	}
+    }
 }
 
 
