@@ -1595,11 +1595,10 @@ static xmlSAXHandler glade_options_parser = {
 
 
 
-gboolean
+void
 glade_project_load_options (GladeProject *project)
 {
   gchar *filename, *base;
-  gboolean retval = FALSE;
 
   /* Check if the options file exists. If it doesn't we use defaults for
      everything. */
@@ -1617,10 +1616,6 @@ glade_project_load_options (GladeProject *project)
 	{
 	  g_warning("document not well formed!");
 	}
-      else
-	{
-	  retval = TRUE;
-	}
     }
 
   /* Check that the directory options are set to defaults, but only if we
@@ -1639,8 +1634,6 @@ glade_project_load_options (GladeProject *project)
 
   g_free (filename);
   g_free (base);
-
-  return retval;
 }
 
 
