@@ -450,6 +450,14 @@ glade_project_options_init (GladeProjectOptions * options)
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), c_options_vbox, label);
   gtk_container_set_border_width (GTK_CONTAINER (c_options_vbox), 7);
 
+  label = gtk_label_new (NULL);
+  gtk_label_set_markup (GTK_LABEL (label), _("<b>Note:</b> for large applications the use of libglade is recommended."));
+  gtk_widget_set_size_request (label, 400, -1);
+  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+  gtk_widget_show (label);
+  gtk_box_pack_start (GTK_BOX (c_options_vbox), label, FALSE, TRUE, 4);
+
   hbox = gtk_hbox_new (TRUE, 4);
   gtk_widget_show (hbox);
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
@@ -702,6 +710,16 @@ glade_project_options_init (GladeProjectOptions * options)
 
   row = 0;
 
+  label = gtk_label_new (NULL);
+  gtk_label_set_markup (GTK_LABEL (label), _("<b>Note:</b> this option is deprecated - use intltool instead."));
+  gtk_widget_set_size_request (label, 400, -1);
+  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+  gtk_widget_show (label);
+  gtk_table_attach (GTK_TABLE (table), label,
+		    0, 3, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+  row++;
+
   /* Output translatable strings. */
   options->output_translatable_strings = gtk_check_button_new_with_label (_("Save Translatable Strings"));
   gtk_widget_show (options->output_translatable_strings);
@@ -1094,7 +1112,7 @@ glade_project_options_generate_program_name (GladeProjectOptions *options)
 {
   gchar *project_name, *program_name;
 
-  (char*) project_name = gtk_entry_get_text (GTK_ENTRY (options->name_entry));
+  project_name = (char*) gtk_entry_get_text (GTK_ENTRY (options->name_entry));
   program_name = glade_project_options_generate_program_name_from_project_name (project_name);
   options->auto_generation_level++;
   gtk_entry_set_text (GTK_ENTRY (options->program_name_entry), program_name);
