@@ -45,6 +45,13 @@ static gchar *Resizable = "GnomePropertyBox|GtkWindow::resizable";
 static gchar *DestroyWithParent = "GnomePropertyBox|GtkWindow::destroy_with_parent";
 static gchar *Icon = "GnomePropertyBox|GtkWindow::icon";
 
+static gchar *Role = "GnomePropertyBox|GtkWindow::role";
+static gchar *TypeHint = "GnomePropertyBox|GtkWindow::type_hint";
+static gchar *SkipTaskbar = "GnomePropertyBox|GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "GnomePropertyBox|GtkWindow::skip_pager_hint";
+static gchar *Decorated = "GnomePropertyBox|GtkWindow::decorated";
+static gchar *Gravity = "GnomePropertyBox|GtkWindow::gravity";
+
 static void show_gnome_property_box_dialog (GbWidgetNewData * data);
 static void on_gnome_property_box_dialog_ok (GtkWidget * widget,
 					     GbWidgetNewData * data);
@@ -211,7 +218,9 @@ gb_gnome_property_box_create_properties (GtkWidget * widget, GbWidgetCreateArgDa
 		  DefaultWidth, DefaultHeight,
 		  Shrink, Grow, AutoShrink,
 		  WMName, WMClass,
-		  Resizable, DestroyWithParent, Icon);
+		  Resizable, DestroyWithParent, Icon,
+		  Role, TypeHint, SkipTaskbar,
+		  SkipPager, Decorated, Gravity);
 }
 
 
@@ -230,7 +239,9 @@ gb_gnome_property_box_get_properties (GtkWidget *widget, GbWidgetGetArgData * da
 		DefaultWidth, DefaultHeight,
 		Shrink, Grow, AutoShrink,
 		WMName, WMClass,
-		Resizable, DestroyWithParent, Icon);
+		Resizable, DestroyWithParent, Icon,
+		Role, TypeHint, SkipTaskbar,
+		SkipPager, Decorated, Gravity);
 }
 
 
@@ -249,7 +260,9 @@ gb_gnome_property_box_set_properties (GtkWidget * widget, GbWidgetSetArgData * d
 		DefaultWidth, DefaultHeight,
 		Shrink, Grow, AutoShrink,
 		WMName, WMClass,
-		Resizable, DestroyWithParent, Icon);
+		Resizable, DestroyWithParent, Icon,
+		Role, TypeHint, SkipTaskbar,
+		SkipPager, Decorated, Gravity);
 }
 
 
@@ -292,7 +305,9 @@ gb_gnome_property_box_write_source (GtkWidget * widget, GbWidgetWriteSourceData
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
 				   WMName, WMClass,
-				   Resizable, DestroyWithParent, Icon);
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
 
   /* We output the source code for the children here, since the code should
      not include calls to create the widgets. We need to specify that the
