@@ -21,9 +21,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifndef _WIN32
 #include <unistd.h>
-#endif
 #include <dirent.h>
 #include <errno.h>
 
@@ -64,8 +62,6 @@ static GtkWidget* glade_util_find_box_accelerator_target (GtkWidget *parent,
 							  GtkWidget *child);
 static GtkWidget* glade_util_find_fixed_accelerator_target (GtkWidget *parent,
 							    GtkWidget *child);
-static GtkWidget* glade_util_find_toolbar_accelerator_target (GtkWidget *parent,
-							      GtkWidget *child);
 static GtkWidget* glade_util_find_layout_accelerator_target (GtkWidget *parent,
 							     GtkWidget *child);
 
@@ -652,8 +648,6 @@ glade_util_find_default_accelerator_target	(GtkWidget	   *label)
 	target = glade_util_find_box_accelerator_target (parent, child);
       else if (GTK_IS_FIXED (parent))
 	target = glade_util_find_fixed_accelerator_target (parent, child);
-      else if (GTK_IS_TOOLBAR (parent))
-	target = glade_util_find_toolbar_accelerator_target (parent, child);
       else if (GTK_IS_LAYOUT (parent))
 	target = glade_util_find_layout_accelerator_target (parent, child);
 
@@ -819,27 +813,6 @@ glade_util_find_fixed_accelerator_target (GtkWidget *parent, GtkWidget *child)
 }
 
 
-/* This tries to find the widget on the right of the given child. */
-static GtkWidget*
-glade_util_find_toolbar_accelerator_target (GtkWidget *parent,
-					    GtkWidget *child)
-{
-  GtkToolbarChild *tchild;
-  GList *elem;
-  gint pos;
-
-  tchild = glade_util_find_toolbar_child (parent, child, &pos, &elem);
-  g_return_val_if_fail (tchild != NULL, NULL);
-
-  elem = elem->next;
-  if (elem == NULL)
-    return NULL;
-
-  tchild = (GtkToolbarChild*) elem->data;
-  return tchild->widget;
-}
-
-
 /* This tries to find the widget on the right of the given child. */
 static GtkWidget*
 glade_util_find_layout_accelerator_target (GtkWidget *parent, GtkWidget *child)
@@ -1009,35 +982,6 @@ glade_util_find_packer_child (GtkPacker * packer,
 #endif
 
 
-/* This finds the toolbar structure representing the given child, or NULL
-   if it is not found. If pos or elem are non-NULL, they return the position
-   and list element of the child as well. */
-GtkToolbarChild*
-glade_util_find_toolbar_child (GtkWidget *toolbar,
-			       GtkWidget *child,
-			       gint *pos,
-			       GList **elem)
-{
-  GList *item;
-  gint position = 0;
-
-  for (item = GTK_TOOLBAR (toolbar)->children; item; item = item->next)
-    {
-      GtkToolbarChild *tbchild = (GtkToolbarChild*) item->data;
-      if (tbchild->widget == child)
-	{
-	  if (pos)
-	    *pos = position;
-	  if (elem)
-	    *elem = item;
-	  return tbchild;
-	}
-      position++;
-    }
-  return NULL;
-}
-
-
 /* This returns the GtkLabel's text, including the underline characters.
    We don't really need it now for GTK+ 2.0. Free the returned string. */
 gchar*
@@ -1653,26 +1597,17 @@ glade_util_get_next_free_project_directory (gchar **project_directory_return,
 {
   GladeError *error = NULL;
   gchar *projects_dir, *project_string, *subdir;
-  const gchar *home_dir;
   DIR *directory;
   struct dirent *entry;
   gint project_num, max_project_num, project_string_len;
   gint num_matched, chars_matched;
 
-  home_dir = g_get_home_dir ();
-  if (home_dir)
-    {
-      projects_dir = glade_util_make_absolute_path (home_dir, _("Projects"));
-    }
-  else
-    {
 #ifdef _WIN32
-      projects_dir = g_strdup ("C:\\Projects");
+  projects_dir = "C:\\Projects";
 #else
-      /* Just use the root directory. Though it probably won't be writable. */
-      projects_dir = g_strdup (G_DIR_SEPARATOR_S);
+  projects_dir = glade_util_make_absolute_path (g_get_home_dir (),
+						_("Projects"));
 #endif
-    }
 
   /* Step through the 'Projects' directory, if it exists, to find
      subdirectories named 'projectXX', and get the highest number used so
@@ -2238,3 +2173,74 @@ glade_util_fopen (const gchar *filename_utf8, const gchar *mode)
   g_free (on_disk_filename);
   return fp;
 }
+
+
+/**
+ * glade_util_uri_list_parse:
+ * @uri_list: the text/urilist, must be NULL terminated.
+ *
+ * Extracts a list of file names from a standard text/uri-list,
+ * such as one you would get on a drop operation.
+ * This is mostly stolen from gnome-vfs-uri.c.
+ *
+ * Returns a GList of gchars.
+ **/
+GList *
+glade_util_uri_list_parse (const gchar *uri_list)
+{
+  const gchar *p, *q;
+  GList *result = NULL;
+
+  g_return_val_if_fail (uri_list != NULL, NULL);
+
+  p = uri_list;
+
+  /* We don't actually try to validate the URI according to RFC
+   * 2396, or even check for allowed characters - we just ignore
+   * comments and trim whitespace off the ends.  We also
+   * allow LF delimination as well as the specified CRLF.
+   */
+  while (p)
+    {
+      if (*p != '#')
+        {
+          gchar *retval;
+          gchar *path;
+
+          while (g_ascii_isspace (*p))
+            p++;
+
+          q = p;
+          while ((*q != '\0') && (*q != '\n') && (*q != '\r'))
+            q++;
+
+          if (q > p)
+            {
+              q--;
+              while (q > p && g_ascii_isspace (*q))
+                q--;
+
+              retval = g_new (gchar, q - p + 2);
+              memcpy (retval, p, q - p + 1);
+              retval[q - p + 1] = '\0';
+
+              path = g_filename_from_uri (retval, NULL, NULL);
+              if (!path)
+                {
+                  g_free (retval);
+                  continue;
+                }
+
+              result = g_list_prepend (result, path);
+
+              g_free (retval);
+            }
+        }
+      p = strchr (p, '\n');
+      if (p)
+        p++;
+    }
+
+  return g_list_reverse (result);
+}
+
